// SENSORCALIBRATION
#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <stdbool.h>
#include <string.h>
#include <math.h>

#include <avr/sleep.h>
#include <avr/io.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <util/setbaud.h>


#define WaitCOL_MATRIX 20
#define WaitROW_MATRIX 6
#define SINF_MATRIX 7
#define SINA_MATRIX 54 
#define PWR_MATRIX 0x1f 

#define WaitCOL_SLIDER 20
#define WaitROW_SLIDER 6
#define SINF_SLIDER 7
#define SINA_SLIDER 28
#define PWR_SLIDER 0x1f 

//#define GCO_DACREF_MATRIX 0x62 

#define WaitCOL_S 20
#define WaitROW_S 6
#define SINF_S 7
#define SINA_S 63 
#define PWR_S 0x1f 

#define GCO_DACREF_S 0x62 
#define COLEND_ROWEND_S 0x26  
#define COLSTART_ROWSTART_S 0x21

#define COLEND_ROWEND_S1 0x16 
#define COLSTART_ROWSTART_S1 0x13

#define GCO_DACREF_S2 0x64
#define COLEND_ROWEND_S2 0x12
#define COLSTART_ROWSTART_S2 0x12

#define GCO_DACREF_S3 0x66
#define COLEND_ROWEND_S3 0x59
#define COLSTART_ROWSTART_S3 0x59

//MATRIX
#define GCO_DACREF_MATRIX 0x66
#define PCO_MATRIX 131
#define COLSTART_ROWSTART_MATRIX 0x06
#define COLEND_ROWEND_MATRIX 0x39

//SLIDER
#define GCO_DACREF_SLIDER 0x66
#define PCO_SLIDER 124
#define COLSTART_ROWSTART_SLIDER 0x40
#define COLEND_ROWEND_SLIDER 0x45


#define PCO_S 170
#define START_5004 0x01


// SPI
#define DD_MOSI    PORTB3
#define DD_SCK     PORTB5
#define DDR_SPI    DDRB
#define SPI_PORT    PORTB
#define SPI_SEL_PIN PORTB2

//sensor processing

#define INT_PORT PINB
#define INT PINB1
#define thresh 3 //touch threshold
#define hyst 1 // hysteresis
#define thresh_neg 10   // negative threshold for baseline update
#define neg_count 10 // min number of negative pulses for baseline update

#define pulsewait 5     //EMI Protektion - min number of pulses for touch recognition
#define baseupdate0 5// baseline update time in 32ms
#define baseupdate1 30
#define baseupdate2 30
#define baseupdate3 100
#define base_t 5  // maximum allowed delta (+-) for baselineupdate
#define mode_basemul 10 // baseline update time multiplikator for mode-button

// LED

//#define LED_L PORTC  
//#define LED_R PORTD
#define PWM_STEPS 40

#define LED_MATRIX_ON DDRC
#define LED_MATRIX PORTC
#define LED_SEVEN_SEG_ON DDRD
#define LED_SEVEN_SEG PORTD
#define LED_1  0x00
#define LED_2  0x01
#define LED_3  0x02
#define LED_4  0x03
#define LED_5  0x04
#define LED_6  0x05
#define LED_7  0x06
#define LED_8  0x07
#define LED_9  0x08
#define LED_10  0x09
#define LED_11  0x0a
#define LED_12  0x0b
#define LED_13  0x0c
#define LED_14  0x0d
#define LED_15  0x0e
#define LED_16  0x0f



// jrgs macros
#define BIT(bit)              (1 << bit)

#define SBI(reg, bit)         reg |= BIT(bit)
#define CBI(reg, bit)         reg &= ~BIT(bit)

#define BIS(reg, bit)         reg & BIT(bit)
#define BIC(reg, bit)         !(reg & BIT(bit))

#define WAIT_BIS(reg, bit)    while(BIC(reg, bit))
#define WAIT_BIC(reg, bit)    while(BIS(reg, bit))

#define LIMIT(x, min, max)    (x < min) ? min : (x > max) ? max : x
