#include "defines.h"
#include "main.h"
#include "spi.h"

void SPI_MasterInit(void)
{
    // Set MOSI , SEL and SCK output, all others input
    DDR_SPI = (1<<SPI_SEL_PIN)|(1<<DD_MOSI)|(1<<DD_SCK);
    // Enable SPI, Master, set clock rate fck/8
    SPCR = (1<<SPE)|(1<<MSTR)|(1<<SPR0);
    SPSR = (1<<SPI2X);
    SPI_PORT |= (1<<SPI_SEL_PIN);
}


void SPI_MasterTransmit(uint8_t cData)
{
    /* Start transmission */
    SPDR = cData;
    /* Wait for transmission complete */
    while(!(SPSR & (1<<SPIF)));
}


// Sending controll byte + 3 data bytes to 5004 with enabling/disabling SPI
void SPI_SEND_5004(uint8_t adress, uint8_t send2 ,uint8_t send1 ,uint8_t send0)
{

    _delay_us(15);
   SPI_PORT &= ~(1<<SPI_SEL_PIN);
   _delay_us(2);
    
    SPI_MasterTransmit(adress);  //send adress 
    SPI_MasterTransmit(send2);  //sending bit 23-16

    SPI_MasterTransmit(send1);  //sending bit 15-8
    SPI_MasterTransmit(send0);  //sending bit 7-0
    
   SPI_PORT |= (1<<SPI_SEL_PIN);
   _delay_us(2);

 }

//Loading all register values needet for matrix sensor measurement 
void LOAD_CONFIG_5004(void)
 {
    
   SPI_READ_5004_internal_sensors(); //dummy read to assure the PE5004 interrupt is low
   SPI_SEND_5004( 0x02,(WaitCOL_S>>4),(WaitCOL_S<<4 | WaitROW_S >>8),WaitROW_S);
   SPI_SEND_5004( 0x03,(SINA_S>>4 | SINF_S<<2),(PWR_S>>1 | SINA_S<<4),(GCO_DACREF_S | PWR_S<<7));
   SPI_SEND_5004( 0x04,0x00,(PCO_S >>8),PCO_S);

   
}

// processing ADC-Value from PE5004 after H-Signal from Pin INT
void SPI_READ_5004_internal_sensors(void) 
{
    
   SPI_PORT &= ~(1<<SPI_SEL_PIN);
    _delay_us(2);

    SPDR = 0x00;                    //adress 0
    while(!(SPSR & (1<<SPIF)));
    SPDR = 0x00; 
    while(!(SPSR & (1<<SPIF)));
    COL_ROW = SPDR;                 //read sensor position
    SPDR = 0x00; 
    while(!(SPSR & (1<<SPIF)));
    SAR_DATA = (SPDR<<4); // upper 6bits of SAR-DATA
    SPDR = 0x00; 
    while(!(SPSR & (1<<SPIF)));        
    SAR_DATA |= (SPDR>>4); // lower 4bits of SAR-DATA
    
    SPI_PORT |= (1<<SPI_SEL_PIN);
   _delay_us(2);
}

    
