#include "defines.h"
#include "terminal.h"

void terminal_init() {
    UBRRH = UBRRH_VALUE;
   UBRRL = UBRRL_VALUE;   

    #if USE_2X
        SBI(UCSRA, U2X);
    #else
        CBI(UCSRA, U2X);
    #endif

    UCSRB = BIT(RXEN) | BIT(TXEN);
    UCSRC = BIT(URSEL) | BIT(UCSZ1) | BIT(UCSZ0); // 8 data, no parity, 1 stop
}

void terminal_clrkey() {
    while(BIS(UCSRA, RXC)) UDR;
}

char terminal_getkey() {
    if(BIC(UCSRA, RXC)) return 0;
    return UDR;
}

char terminal_getc() {
    WAIT_BIS(UCSRA, RXC);
    return UDR;
}

void terminal_putc(char c) {
    WAIT_BIS(UCSRA, UDRE);
    UDR = c;
}

void terminal_gets(char *s, uint8_t size) {
    char c;
    uint8_t cnt = 0;

    while(BIS(UCSRA, RXC)) UDR;

    do {
        c = terminal_getc();
        if(c >= 32 && c <= 126) {
            if(cnt < size) {
                terminal_putc(c);
                *s++ = c; cnt++;
            } else {
                terminal_putc(0x07);
            }
        }
        if(c == 0x08) {
            if(cnt > 0) {
                terminal_puts("\x08 \x08");
                *s-- = 0; cnt--;
            } else {
                terminal_putc(0x07);
            }
        }
    } while(c != 0x0D);

    *s = 0;
}

void terminal_puts(char *s) {
    char c;

    while((c = *s)) {
        if(c == 0x0A) terminal_putc(0x0D);
        terminal_putc(c); s++;
    }
}

void terminal_puts_P(char *s) {
    char c;

    while((c = pgm_read_byte(s))) {
        if(c == 0x0A) terminal_putc(0x0D);
        terminal_putc(c); s++;
    }
}

void terminal_clearscreen() {
    terminal_puts("\x1B[m");
    terminal_puts("\x1B[2J");
    terminal_puts("\x1B[H");
}

void terminal_clearline() {
    terminal_puts("\x1B[2K");
    terminal_puts("\x0D");
}

void terminal_setpos(uint8_t x, uint8_t y) {
    char buf[4];

    terminal_puts("\x1B[");
    terminal_puts(utoa(y, buf, 10));
    terminal_puts(";");
    terminal_puts(utoa(x, buf, 10));
    terminal_puts("H");
}

void terminal_setcolor(uint8_t bright, uint8_t color) {
    char buf[4];

    terminal_puts("\x1B[");
    terminal_puts(utoa(bright, buf, 10));
    terminal_puts(";");
    terminal_puts(utoa(color, buf, 10));
    terminal_puts("m");
}





